@@ -76,6 +76,7 @@ my $nokia_7210c = $server =~ /nokia_7210c/;
 my $ovi = $server =~ /Ovi/;
 my $unique_uid = $ENV{CLIENT_TEST_UNIQUE_UID};
 my $full_timezones = $ENV{CLIENT_TEST_FULL_TIMEZONES}; # do not simplify VTIMEZONE definitions
+my $no_timezones =  $ENV{CLIENT_TEST_NO_TIMEZONES};
 
 # TODO: this hack ensures that any synchronization is limited to
 # properties supported by Synthesis. Remove this again.
@@ -86,6 +87,7 @@ my $egroupware = $server =~ /egroupware/;
 my $funambol = $server =~ /funambol/;
 my $googlesyncml = $server eq "google";
 my $googlecaldav = $server eq "googlecalendar";
+my $googlecarddav = $server eq "googlecontacts";
 my $googleeas = $server eq "googleeas";
 my $google_valarm = $ENV{CLIENT_TEST_GOOGLE_VALARM};
 my $yahoo = $server =~ /yahoo/;
@@ -93,6 +95,7 @@ my $davical = $server =~ /davical/;
 my $apple = $server =~ /apple/;
 my $oracle = $server =~ /oracle/;
 my $radicale = $server =~ /radicale/;
+my $zimbra = $server =~ /zimbra/;
 my $evolution = $client =~ /evolution/;
 my $addressbook = $client =~ /addressbook/;
 
@@ -164,6 +167,15 @@ sub NormalizeItem {
     my $width = shift;
     $_ = shift;
 
+    # Reduce \N to \n (both are allowed in vCard 3.0).
+    # Using a regular expression is a bit too broad
+    # because it also matches \\N, which must not be
+    # changed.
+    s/\\N/\\n/g;
+
+    # Ignore blank lines. Akonadi inserts them.
+    s/\n{2,}/\n/s;
+
     # undo line continuation
     s/\n\s//gs;
     # ignore charset specifications, assume UTF-8
@@ -198,11 +210,56 @@ sub NormalizeItem {
     # make TYPE uppercase (in vCard 3.0 at least those parameters are case-insensitive)
     while( s/^(\w*[^:\n]*);TYPE=(\w*?[a-z]\w*?)([;:])/ $1 . ";TYPE=" . uppercase($2) . $3 /mge ) {}
 
-    # replace parameters with a sorted parameter list
-    s!^([^;:\n]*);(.*?):!$1 . ";" . join(';',sort(split(/;/, $2))) . ":"!meg;
+    # Replace parameters with a sorted parameter list. Cannot be done with
+    # a regular expression because of quoted strings. While we know exact
+    # parameter values, normalize them to use quoted strings if and only
+    # if the content is more complex than alphanumeric plus underscore and
+    # hyphen.
+    my @lines;
+    my ($propname, $sep, $rest);
+    foreach (split /\n/) {
+        ($propname, $sep, $rest) = /^([^;:]+)([:;])(.*)/;
+        if ($sep eq ";") {
+            my @params;
+            my $c;
+            my $i = 0;
+            my $n = length($rest);
+            my $quoted = 0;
+            my $start = 0;
+            while ($i < $n) {
+                $c = substr($rest, $i, 1);
+                $i++;
+                if ($quoted) {
+                    if ($c eq '"') {
+                        $quoted = 0;
+                    }
+                } else {
+                    if ($c eq '"') {
+                        $quoted = 1;
+                    } elsif ($c eq ';' || $c eq ':') {
+                        my $param = substr($rest, $start, $i - $start - 1);
+                        my ($name, $value) = $param =~ /^([^=]*)="?([^"]*)"?$/;
+                        if ($value =~ /^[a-zA-Z0-9_-]*$/) {
+                            $param = $name . '=' . $value;
+                        } else {
+                            $param = $name . '="' . $value . '"';
+                        }
+                        push @params, $param;
+                        $start = $i;
+                        if ($c eq ':') {
+                            last;
+                        }
+                    }
+                }
+            }
+            $_ = $propname . ';' . join(";", sort(@params)) . ':' . substr($rest, $start);
+        }
+        push @lines, $_;
+    }
+    $_ = join("\n", @lines);
 
-    # EXDATE;VALUE=DATE is the default, no need to show it
-    s/^EXDATE;VALUE=DATE:/EXDATE:/mg;
+    # VALUE=DATE is the default, no need to show it
+    s/^(EXDATE|BDAY);VALUE=DATE:/\1:/mg;
 
     # default opacity is OPAQUE
     s/^TRANSP:OPAQUE\r?\n?//gm;
@@ -273,8 +330,9 @@ sub NormalizeItem {
 
     # remove fields which may differ
     s/^(PRODID|CREATED|DTSTAMP|LAST-MODIFIED|REV)(;X-VOBJ-FLOATINGTIME-ALLOWED=(TRUE|FALSE))?:.*\r?\n?//gm;
-    # remove optional fields
-    s/^(METHOD|X-WSS-[A-Z]*|X-WR-[A-Z]*|CALSCALE):.*\r?\n?//gm;
+    # remove optional properties and parameters
+    s/^(METHOD|X-WSS-[A-Z]*|X-WR-[A-Z]*|CALSCALE|X-KDE-ICAL-IMPLEMENTATION-VERSION|X-KDE-KCALCORE-ENABLED):.*\r?\n?//gm;
+    s/^(ATTENDEE[^:]*);X-UID=[^;:]*/$1/mg;
 
     # trailing line break(s) in a DESCRIPTION may or may not be
     # removed or added by servers
@@ -315,11 +373,13 @@ sub NormalizeItem {
     # added by Google CalDAV server when storing an all-day event
     # which doesn't need any time zone definition)
     # http://code.google.com/p/google-caldav-issues/issues/detail?id=63
+    #
+    # Also strip all definitions if requested.
     while (m/(BEGIN:VTIMEZONE.*?TZID:([^\n]*)\n.*?END:VTIMEZONE\n)/gs) {
         my $def = $1;
         my $tzid = $2;
-        # used as parameter?
-        if (! m/;TZID="?\Q$tzid\E"?/) {
+        # Strip all, or not used as parameter?
+        if ($no_timezones || ! m/;TZID="?\Q$tzid\E"?/) {
             # no, remove definition
             s!\Q$def\E!!s;
         }
@@ -329,7 +389,9 @@ sub NormalizeItem {
         # Strip trailing digits from TZID. They are appended by
         # Evolution and SyncEvolution to distinguish VTIMEZONE
         # definitions which have the same TZID, but different rules.
-        s/(^TZID:|;TZID=)([^;:]*?) \d+/$1$2/gm;
+        # This appending of digits may even get repeated, leading to:
+        # TZID=EST/EDT 1 1
+        s/(^TZID:|;TZID=)([^;:]*?)( \d+)+/$1$2/gm;
 
         # Strip trailing -(Standard) from TZID. Evolution 2.24.5 adds
         # that (not sure exactly where that comes from).
@@ -354,6 +416,10 @@ sub NormalizeItem {
         # VALUE=DURATION is the default behavior
         s/^TRIGGER([^\n:]*);VALUE=DURATION/TRIGGER$1/mg;
         s/^(TRIGGER.*):(\S*)/$1 . ":" . NormalizeTrigger($2)/mge;
+        # INDIVIDUAL is default for CUTYPE.
+        s/;CUTYPE=INDIVIDUAL([;:])/$1/mg;
+        # Print without quotation marks (probably not save in general, but okay for our test data).
+        s/;CN="([^;]*)"/;CN=$1/g;
     }
 
     # Added by EDS >= 2.32, presumably to cache some internal computation.
@@ -365,7 +431,7 @@ sub NormalizeItem {
     #                                      >    LY                                 
     s/^(\w+)([^:\n]*);X-EVOLUTION-ENDDATE=[0-9TZ]*/$1$2/mg;
 
-    if ($scheduleworld || $egroupware || $synthesis || $addressbook || $funambol ||$googlesyncml || $googleeas || $mobical || $memotoo) {
+    if ($scheduleworld || $egroupware || $synthesis || $addressbook || $funambol ||$googlesyncml || $googleeas || $googlecarddav || $mobical || $memotoo || $zimbra) {
       # does not preserve X-EVOLUTION-UI-SLOT=
       s/^(\w+)([^:\n]*);X-EVOLUTION-UI-SLOT=\d+/$1$2/mg;
     }
@@ -391,23 +457,30 @@ sub NormalizeItem {
       s/^(TEL.*);TYPE=PREF/$1/mg;
     }
 
-   if($googlesyncml || $googleeas) {
+   if($googlesyncml || $googleeas || $googlecarddav) {
       # ignore the PHOTO encoding data
       s/^PHOTO(.*?): .*\n/PHOTO$1: [...]\n/mg;
    }
 
-   if($googlesyncml) {
-      # FN propertiey is not correct 
+   if($googlesyncml || $googlecarddav) {
+      # FN property gets synthesized by Google.
       s/^FN:.*\n/FN$1: [...]\n/mg;
+   }
+
+   # Properties and parameters are case-insensitive. ownCloud uses
+   # X-ABLABEL while everyone else uses X-ABLabel.
+   s/X-ABLABEL/X-ABLabel/g;
+
+   if ($googlesyncml) {
       # Not support car type in telephone
       s!^TEL\;TYPE=CAR(.*)\n!TEL$1\n!mg;
       # some properties are lost
-      s/^(X-EVOLUTION-FILE-AS|NICKNAME|BDAY|CATEGORIES|CALURI|FBURL|ROLE|URL|X-AIM|X-EVOLUTION-UI-SLOT|X-ANNIVERSARY|X-ASSISTANT|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GROUPWISE|X-ICQ|X-GADUGADU|X-JABBER|X-MSN|X-SIP|X-SKYPE|X-MANAGER|X-SPOUSE|X-MOZILLA-HTML|X-YAHOO)(;[^:;\n]*)*:.*\r?\n?//gm;
+      s/^(X-EVOLUTION-FILE-AS|NICKNAME|BDAY|CATEGORIES|CALURI|FBURL|GEO|ROLE|URL|X-AIM|X-EVOLUTION-UI-SLOT|X-ANNIVERSARY|X-ASSISTANT|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GROUPWISE|X-ICQ|X-GADUGADU|X-JABBER|X-MSN|X-SIP|X-SKYPE|X-MANAGER|X-SPOUSE|X-MOZILLA-HTML|X-YAHOO)(;[^:;\n]*)*:.*\r?\n?//gm;
    }
 
    if ($googlecaldav) {
       #several properties are not preserved by Google in icalendar2.0 format
-      s/^(SEQUENCE|X-EVOLUTION-ALARM-UID)(;[^:;\n]*)*:.*\r?\n?//gm;
+      s/^(SEQUENCE|X-EVOLUTION-ALARM-UID|TRANSP)(;[^:;\n]*)*:.*\r?\n?//gm;
 
       # Google adds calendar owner as attendee of meetings, regardless
       # whether it was on the original attendee list. Ignore this
@@ -441,10 +514,8 @@ sub NormalizeItem {
       s/^STATUS:CONFIRMED\r?\n?//gm;
     }
 
-    # Google randomly (?!) adds a standard alarm to events.
-    if ($google_valarm) {
-        s/BEGIN:VALARM\nDESCRIPTION:This is an event reminder\nACTION:DISPLAY\nTRIGGER:-PT10M\n(X-KDE-KCALCORE-ENABLED:TRUE\n)END:VALARM\n//s;
-    }
+    # Ignore VALARM ACTION:NONE. This has to be added to avoid default alarms in Google CalDAV.
+    s/^BEGIN:VALARM\r?\n.*?^ACTION:NONE\r?\n.*?^END:VALARM\r?\n//ms;
 
     if ($yahoo) {
         s/^(X-MICROSOFT-[-A-Z0-9]*)(;[^:;\n]*)*:.*\r?\n?//gm;
@@ -464,7 +535,7 @@ sub NormalizeItem {
 
     if ($synthesis) {
       # does not preserve certain properties
-      s/^(FN|BDAY|X-MOZILLA-HTML|X-EVOLUTION-FILE-AS|X-AIM|NICKNAME|UID|PHOTO|CALURI|SEQUENCE|TRANSP|ORGANIZER|ROLE|FBURL|X-ANNIVERSARY|X-ASSISTANT|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GADUGADU|X-GROUPWISE|X-ICQ|X-JABBER|X-MANAGER|X-MSN|X-SIP|X-SKYPE|X-SPOUSE|X-YAHOO)(;[^:;\n]*)*:.*\r?\n?//gm;
+      s/^(FN|GEO|BDAY|X-MOZILLA-HTML|X-EVOLUTION-FILE-AS|X-AIM|NICKNAME|UID|PHOTO|CALURI|SEQUENCE|TRANSP|ORGANIZER|ROLE|FBURL|X-ANNIVERSARY|X-ASSISTANT|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GADUGADU|X-GROUPWISE|X-ICQ|X-JABBER|X-MANAGER|X-MSN|X-SIP|X-SKYPE|X-SPOUSE|X-YAHOO)(;[^:;\n]*)*:.*\r?\n?//gm;
       # default ADR is HOME
       s/^ADR;TYPE=HOME/ADR/gm;
       # only some parts of N are preserved
@@ -490,7 +561,7 @@ sub NormalizeItem {
 
     if ($funambol) {
       # several properties are not preserved
-      s/^(CALURI|FBURL|X-MOZILLA-HTML|X-EVOLUTION-FILE-AS|X-AIM|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GROUPWISE|X-ICQ|X-YAHOO|X-GADUGADU|X-JABBER|X-MSN|X-SIP|X-SKYPE|X-ASSISTANT)(;[^:;\n]*)*:.*\r?\n?//gm;
+      s/^(CALURI|FBURL|GEO|X-MOZILLA-HTML|X-EVOLUTION-FILE-AS|X-AIM|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GROUPWISE|X-ICQ|X-YAHOO|X-GADUGADU|X-JABBER|X-MSN|X-SIP|X-SKYPE|X-ASSISTANT)(;[^:;\n]*)*:.*\r?\n?//gm;
 
       # quoted-printable line breaks are =0D=0A, not just single =0A
       s/(?<!=0D)=0A/=0D=0A/g;
@@ -600,7 +671,7 @@ sub NormalizeItem {
     }
     if ($memotoo) {
       if (/^BEGIN:VCARD/m ) {
-        s/^(FN|FBURL|CALURI|ROLE|X-MOZILLA-HTML|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GADUGADU|X-JABBER|X-MSN|X-SIP|X-SKYPE|X-GROUPWISE)(;[^:;\n]*)*:.*\r?\n?//gm;
+        s/^(FN|FBURL|GEO|CALURI|ROLE|X-MOZILLA-HTML|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GADUGADU|X-JABBER|X-MSN|X-SIP|X-SKYPE|X-GROUPWISE)(;[^:;\n]*)*:.*\r?\n?//gm;
         # s/^(FN|FBURL|CALURI|CATEGORIES|ROLE|X-MOZILLA-HTML|X-EVOLUTION-FILE-AS|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GADUGADU|X-JABBER|X-MSN|X-SIP|X-SKYPE|X-GROUPWISE)(;[^:;\n]*)*:.*\r?\n?//gm;
         # strip 'TYPE=HOME' 
         s/^URL([^\n:]*);TYPE=HOME/URL$1/mg;
@@ -625,7 +696,7 @@ sub NormalizeItem {
       }
     }
     if ($mobical) {
-      s/^(CALURI|CATEGORIES|FBURL|NICKNAME|X-MOZILLA-HTML|X-EVOLUTION-FILE-AS|X-ANNIVERSARY|X-ASSISTANT|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GROUPWISE|X-ICQ|X-GADUGADU|X-JABBER|X-MSN|X-SIP|X-SKYPE|X-MANAGER|X-SPOUSE|X-YAHOO|X-AIM)(;[^:;\n]*)*:.*\r?\n?//gm;
+      s/^(CALURI|CATEGORIES|FBURL|GEO|NICKNAME|X-MOZILLA-HTML|X-EVOLUTION-FILE-AS|X-ANNIVERSARY|X-ASSISTANT|X-EVOLUTION-BLOG-URL|X-EVOLUTION-VIDEO-URL|X-GROUPWISE|X-ICQ|X-GADUGADU|X-JABBER|X-MSN|X-SIP|X-SKYPE|X-MANAGER|X-SPOUSE|X-YAHOO|X-AIM)(;[^:;\n]*)*:.*\r?\n?//gm;
 
       # some workrounds here for mobical's bug 
       s/^(FN|BDAY)(;[^:;\n]*)*:.*\r?\n?//gm;
@@ -667,6 +738,11 @@ sub NormalizeItem {
         s/^(FN)(;[^:;\n]*)*:.*\r?\n?//gm;
     }
 
+    if ($googleeas || $exchange) {
+        # properties not supported by ActiveSync and/or activesyncd
+        s/^(GEO)(;[^:;\n]*)*:.*\r?\n?//gm;
+    }
+
     if ($googleeas || $exchange) {
         # temporarily ignore modified properties
         s/^(BDAY|X-ANNIVERSARY)(;[^:;\n]*)*:.*\r?\n?//gm;
@@ -692,9 +768,15 @@ sub NormalizeItem {
       }
 
       my $spaces = "  " x ($#formatted - 1);
+
+      # Ignore group tags during folding, add back before indenting.
+      /^([^.:;]+\.)?(.*)/s;
+      my $tag = $1;
+      $_ = $2;
       my $thiswidth = $width -1 - length($spaces);
       $thiswidth = 1 if $thiswidth <= 0;
       s/(.{$thiswidth})(?!$)/$1\n /g;
+      $_ = $tag . $_;
       s/^(.*)$/$spaces$1/mg;
       push @{$formatted[$#formatted]}, $_;
 
@@ -713,15 +795,93 @@ sub NormalizeItem {
           $str =~ /^(\s*)/;
           return length($1);
         }
+        # Sort lines without group tag before lines without group tag.
+        # When both lines have group tags, sort based on line without
+        # group tag, then regroup related items after sorting.
+        sub cmplines {
+            my $a = shift;
+            my $b = shift;
+            $a =~ s/^[^.:;]+\.//;
+            $b =~ s/^[^.:;]+\.//;
+            return $a cmp $b;
+        }
+        my @body;
+        my $isimportant;
+        foreach $_ (@{$block}) {
+            $isimportant = ($_ =~ /^\s*(N|SUMMARY):/);
+            /^(\s*)([^.:;]+\.)?(.*)/s;
+            push @body, [$isimportant, length($1), $2, $3];
+        }
+        my @sorted = sort( { ($a->[1] - $b->[1]) || # Compare indention, more indented last.
+                             ($b->[0] - $a->[0]) || # Compare importance, less important last.
+                             $a->[3] cmp $b->[3] } # Compare property name, parameters and value without group tag.
+                           @body );
+
+        # Combine lines with the same group tag.
+        my %tags;
+        my @tagged;
+        my $tag;
+        my $entry;
+        my $index;
+        foreach (@sorted) {
+            $tag = $_->[2];
+            # Has a line a group tag?
+            if ($tag) {
+                # Same as one found before?
+                $index = $tags{$tag};
+                if (defined($index)) {
+                    # Append to previous instance of the tag, keeping tag indices the same.
+                    push @{$tagged[$index]}, $_;
+                } else {
+                    # Add at end, remember index for next line with the same tag.
+                    push @tagged, $_;
+                    $tags{$tag} = $#tagged;
+                }
+            } else {
+                push @tagged, $_;
+            }
+        }
+
+        # Convert back into individual, indented text lines.
+        my @expanded;
+        foreach (@tagged) {
+            if ($_->[2]) {
+                if ($#{$_} == 4) {
+                    # Simplify IMPP + X-ABLabel:Other to just IMPP without group tag.
+                    # For the sake of simplicity we only do that if the number of
+                    # grouped properties is exactly two. Otherwise we would have
+                    # to search in the list of extra properties.
+                    if ($_->[3] =~ /^IMPP[;:]/ &&
+                        $_->[4][3] =~ /^X-ABLabel:Other$/) {
+                        splice(@{$_}, 4);
+                    }
+                }
+                if ($#{$_} == 3) {
+                    # If the last remaining property is X-ABLabel, then ignore it.
+                    # We ignore empty properties, which can cause their labels to
+                    # be left as redundant information (happens with Google CardDAV
+                    # when sending an empty URL).
+                    if ($_->[3] =~ "X-ABLabel:") {
+                        next;
+                    }
+                    # Remove redundant group tag from other properties.
+                    $_->[2] = "";
+                }
+            }
+            push @expanded, (" " x $_->[1]) . ($_->[2] ? "- " : "") . $_->[3];
+            if ($#{$_} > 3) {
+                foreach ($_->[4,-1]) {
+                    push @expanded, (" " x $_->[1]) . "  " . $_->[3];
+                }
+            }
+        }
+
+        # Create one BEGIN/END block.
         $_ = join("\n",
                   $begin,
-                  sort( { $a =~ /^\s*(N|SUMMARY):/ ? -1 :
-                          $b =~ /^\s*(N|SUMMARY):/ ? 1 :
-                          ($a =~ /^\s/ && $b =~ /^\S/) ? 1 :
-                          numspaces($a) == numspaces($b) ? $a cmp $b :
-                          numspaces($a) - numspaces($b) }
-                        @{$block} ),
+                  @expanded,
                   $end);
+
         push @{$formatted[$#formatted]}, $_;
       }
     }
